package adaption;

import java.util.BitSet;
import java.util.HashMap;
import java.util.Map;

import features.SolverTimesFeature;

/**
 * @author edom
 * Have as a backup in case linear model dosn't work.
 * Now the main model because the linear model dosn't work
 */
public class ThreeSolversSchedule {
	
	private static int time = 120;
	private static int number_of_time_intervals = 60;
	private static double time_unit = 2;
	
	private int nsolved;
	private int number_of_solvers;
	private int[] allocated_time;
	private int[] temp_time;
	private BitSet[][] solved;
	int ndatapoints;
	double[][] datapoints;
	
	double[] weights;
	
	public static void setTime(int new_time){
		time = new_time;
		time_unit = (double)time/(double)number_of_time_intervals;
	}
	
	private SolverTimesFeature[] info;
	
	private void normalizeSimilarities() {
		double themin = Integer.MAX_VALUE;
		double themax = 0.0;
		for(int i=0; i<ndatapoints; ++i) {
			if( weights[i] > themax ) themax = weights[i];
			if( weights[i] < themin ) themin = weights[i];
		}
		if( themax == themin ) {
			for(int i=0; i<ndatapoints; ++i)
				weights[i] = 1.0;
		} else {
			for(int i=0; i<ndatapoints; ++i) {
				weights[i] = ((weights[i] - themin) / (themax - themin));
			}
			System.err.println("c ");
			for(int i=0; i<ndatapoints; ++i) {
				weights[i] = 1/((weights[i] + 1));//*
//						(weights[i] + 1));//*
//						(weights[i] + 1));//TODO: where the weights are set
			}
		}
		System.err.print("c ");
		for(int i=0; i<ndatapoints; ++i) {
			System.err.print( weights[i] + "  " );
		}
		System.err.println();
	}	
	
	private int getScore( BitSet slv ) {
		double score = 0.0;
		for(int i=0; i<ndatapoints; ++i)
			if(slv.get(i))
				score += weights[i];
		return (int)(1000*score);
//		return slv.cardinality();//TODO: change me
	}
	
	public ThreeSolversSchedule(SolverTimesFeature[] info, double[] similarities){
		this.info = info;
		initDataFromInfo();
		
		System.err.println("c Length: " + info.length);
		System.err.print("c ");
		for(int i=0; i<info.length; ++i)
			System.err.print(similarities[i] + "  " );
		System.err.println("");
		this.weights = similarities;
		
		normalizeSimilarities();
		
	}
	
	public ThreeSolversSchedule(SolverTimesFeature[] info){
		System.err.println("c Length: " + info.length);
		this.info = info;
		initDataFromInfo();
	}
	
	public ThreeSolversSchedule(){
	}
	
	private int allocateTimeSolvers( int solver, int[][] steplist, int[] nsteps, 
									 int rtime, BitSet tsolved )
	{
//		for(int x=0; x<solver; ++x)
//			System.err.print( "    " );
//		System.err.println( "solv_" + solver + " " + rtime + " " + tsolved + " " + nsolved );
		
		int i = nsteps[solver];
		//int sizeb=tsolved.cardinality();
		int sizeb=getScore(tsolved);
		
		while( i >= 0 && steplist[solver][i] > rtime ) --i;
		if( i < 0 ) {
			temp_time[solver] = 0;
		} else { 
			if( solver == number_of_solvers-1 )
			{
				temp_time[solver] = steplist[solver][i];
				tsolved.or( solved[solver][temp_time[solver]] );
				//sizeb = tsolved.cardinality();
				sizeb=getScore(tsolved);
			}
			else for(; i>=0; --i)
			{
//				for(int x=0; x<solver; ++x)
//					System.err.print( "    " );
//				System.err.println( "branch on : " + i );
				temp_time[solver] = steplist[solver][i];
				BitSet s = new BitSet(1 + ndatapoints/32);
				s.or( tsolved );
				s.or( solved[solver][temp_time[solver]] );
				allocateTimeSolvers( solver+1, steplist, nsteps, rtime-temp_time[solver], s );
			}
		}
		
		if( sizeb > nsolved )
		{
//			for(int x=0; x<solver; ++x)
//				System.err.print( "    " );
//			System.err.println( "improving solution found: " + tsolved );
			nsolved = sizeb;
			for(i=0; i<=solver; ++i)
				allocated_time[i] = temp_time[i];
			for(i=solver+1; i<number_of_solvers; ++i)
				allocated_time[i] = temp_time[i];
		}
		
		return sizeb;
	}
	
	/**
	 * @return The schedule of solvers to be run
	 */
	public HashMap<String, Integer> getSchedule()
	{
		HashMap<String, Integer> res = new HashMap<String, Integer>();
		
//		initDataFromInfo();
		computeSchedule();
		
		HashMap<Integer, String> names = new HashMap<Integer, String>();
//		names.put(0, "mistral");
//		names.put(1, "abscon");
//		names.put(2, "choco");
		names.put(0, "abscon");
		names.put(1, "mistral");
		names.put(2, "choco");
//		names.put(2, "bprolog");
//		names.put(3, "buggy");
//		names.put(4, "sugar");
		
		for(int i=0; i<number_of_solvers; ++i)
		{
			if( allocated_time[i] > 0 ) {
				System.err.println( "c\t  run " + names.get(i) + " for " +
						(allocated_time[i]*time_unit) + " seconds " + solved[i][allocated_time[i]] + " " 
						+ (getScore(solved[i][allocated_time[i]])) );
				res.put( names.get(i), (int)(allocated_time[i]*time_unit) );
			} 
			else System.err.println( "c\t do not run " + names.get(i) );
		}
		
		return res;
	}
	
	public void initDataFromInfo()
	{
		number_of_solvers = info[0].getValue().size();
		ndatapoints = info.length;
		datapoints = new double[number_of_solvers][ndatapoints];
		for(int i=0; i<number_of_solvers; ++i)
		{
			int j = info.length;
			assert( ndatapoints < 0 || j == ndatapoints );
			while( j-- > 0 ){
				Map<String, Double> times = info[j].getValue();
				datapoints[0][j] = times.get("abscon");
				datapoints[1][j] = times.get("mistral");
				datapoints[2][j] = times.get("choco");//:TODO: change
//				datapoints[2][j] = times.get("bprolog");
//				datapoints[3][j] = times.get("buggy");
//				datapoints[4][j] = times.get("sugar");
			}
		}
	}
		
	public void computeSchedule()
	{
		int i, j, k;
		//double time_unit = (double)time/(double)number_of_time_intervals;

		solved = new BitSet[number_of_solvers][number_of_time_intervals+1];
		boolean[][] steps     = new boolean[number_of_solvers][number_of_time_intervals+1];
		int[][] steplist = new int[number_of_solvers][number_of_time_intervals+1];
		int[]  nsteps     = new int[number_of_solvers];
		
		for(i=0; i<number_of_solvers; ++i) 
			for(j=0; j<=number_of_time_intervals; ++j) {
				solved[i][j] = new BitSet( 1 + ndatapoints/32 );
				steps[i][j] = false;
			}
		for(i=0; i<number_of_solvers; ++i)
		{
			nsteps[i] = 0;
			for(j=0; j<ndatapoints; ++j) 	{
				if( datapoints[i][j] >= 0 && datapoints[i][j] < time ) {
//					System.err.println( datapoints[i][j] + " " + ((int)(datapoints[i][j]/time_unit)) );
					k=(int)(1+datapoints[i][j]/time_unit);
					if( !steps[i][k] ) 
						steps[i][k] = true;
					for(; k<=number_of_time_intervals; ++k) 
						solved[i][k].set(j);
				}
			}
		}
	
		allocated_time = new int[number_of_solvers];
		int total_time = 0;
		
		for(i=0; i<number_of_solvers; ++i)
		{
			steplist[i][0] = 0;
			for(j=1; j<=number_of_time_intervals; ++j) {
				if( steps[i][j] ) 
					steplist[i][++nsteps[i]] = j;
			}
			allocated_time[i] = steplist[i][nsteps[i]];
			total_time += allocated_time[i];
		}
	
		for(i=0; i<number_of_solvers; ++i) {
			System.err.println( "c\nc solver " + i );
			for(j=0; j<=nsteps[i]; ++j)
				System.err.println( "c\t  before " + (steplist[i][j]*time_unit) + " seconds: " 
						+ solved[i][steplist[i][j]] + " " + (getScore(solved[i][steplist[i][j]])) );
			//for(j=0; j<number_of_time_intervals; ++j)
			//	System.err.println( "\t  before " + ((j+1)*time_unit) + " seconds: " + solved[i][j].toString() );
		}
		System.err.println( "c\ncTotal time: " + total_time );
		
		if( total_time <= number_of_time_intervals )
		{
			System.err.println("c " +(total_time*time_unit) + " seconds unused" );	
			int differences[][] = new int[number_of_solvers][number_of_solvers];

			// - differences[i][j] <=> how many instances can we solve using solver i and j rather than j alone

			// - if differences[i][j] is equal to 0, and differences[j][i] is not equal to 0, 
			// then solver i strictly dominates solver j

			// - if differences[i][j] == differences[j][i] == 0 then we say that solver i (resp. j)
			// dominates solver j (resp. i) iff allocated_time[i] <= allocated_time[j]


			for(i=0; i<number_of_solvers; ++i) {
				for(j=0; j<number_of_solvers; ++j) {
					if( i != j )
					{		
						BitSet diff = (BitSet) solved[i][allocated_time[i]].clone();
						differences[i][j] = diff.cardinality();//getScore(diff);
						diff.and(solved[j][allocated_time[j]]);
						differences[i][j] -= diff.cardinality();//getScore(diff);
					}
					System.err.print("c " + differences[i][j] + " " );
				}
				System.err.println();
			}

			// select solvers that are not dominated
			int[] good_solvers = new int[number_of_solvers];
			int ngs = 0, nbs = number_of_solvers;
			boolean isNotDominated;
			for(i=0; i<number_of_solvers; ++i) {
				System.err.println( "c is solver" + i + " dominated by:" );
				isNotDominated = true;
				for(j=0; isNotDominated && j<number_of_solvers; ++j) 
					if( i != j) {
						System.err.println( "c\tsolver " + j + " (" + i + " solves instances that " + j + " can't: " +
								(differences[i][j] != 0) + ") (" + j + " doesn't solve instances that " + i + " can't: " +
								(differences[j][i] == 0) + ") (" + i + " is faster than " + j + ": " +
								(allocated_time[i] < allocated_time[j]) + ") (" + i + " is equally as fast as " + j + ": " +
								(allocated_time[i] == allocated_time[j]) + ") (" + i + " < " + j + ": " + (i > j) + ")" );
						isNotDominated = ( differences[i][j] != 0 || 
								(differences[j][i] == 0 && allocated_time[i] < allocated_time[j]) ||
								(differences[j][i] == 0 && allocated_time[i] == allocated_time[j] && i > j) );
					}
				System.err.println( "c is dominated? " + (!isNotDominated) + "\nc" );
				if( isNotDominated )
					good_solvers[ngs++] = i;
				else
					good_solvers[--nbs] = i;
			}

			System.err.println( "c Number of non-dominated solvers: " + ngs );

			if( ngs > 0 ) {
				System.err.println( "c remove dominated solvers" );
				for(i=0; i<ngs; ++i) 
					System.err.println( "c solver " + good_solvers[i] + " is not dominated by another solver" );
				for(i=ngs; i<number_of_solvers; ++i) {
					System.err.println( "c solver " + good_solvers[i] + " is dominated by another solver" );
					total_time -= allocated_time[good_solvers[i]];
					allocated_time[good_solvers[i]] = 0;
				}
			}

			for(i=0; i<number_of_solvers; ++i)
			{
				System.err.println( "c\nc solver " + i + ": " + (allocated_time[i]*time_unit) );
				if( allocated_time[i] > 0 ) {
					System.err.println( "c\nc+ " + (allocated_time[i]) + "/" 
							+ total_time + " of " + (number_of_time_intervals - total_time) );
					allocated_time[i] += (((number_of_time_intervals - total_time) * allocated_time[i]) / total_time);
				}
			}
		} 
		else 
		{
			temp_time = new int[number_of_solvers];
			BitSet tsolved = new BitSet( 1 + ndatapoints/32 );
			allocateTimeSolvers( 0, steplist, nsteps, number_of_time_intervals, tsolved );
		}
	}
	
	public void testTimeAllocation1() {
		number_of_solvers = 3;
		ndatapoints = 10;
		datapoints = new double[3][10];
		datapoints[0][0] = -1;
		datapoints[0][1] = 450;
		datapoints[0][2] = 10;
		datapoints[0][3] = 1;
		datapoints[0][4] = -1;
		datapoints[0][5] = 20;
		datapoints[0][6] = 500;
		datapoints[0][7] = -1;
		datapoints[0][8] = -1;
		datapoints[0][9] = 1700;
		
		datapoints[1][0] = 450;
		datapoints[1][1] = -1;
		datapoints[1][2] = 10;
		datapoints[1][3] = 1000;
		datapoints[1][4] = -1;
		datapoints[1][5] = -1;
		datapoints[1][6] = 290;
		datapoints[1][7] = 350;
		datapoints[1][8] = -1;
		datapoints[1][9] = -1;
		
		datapoints[2][0] = 1770;
		datapoints[2][1] = 1250;
		datapoints[2][2] = 1700;
		datapoints[2][3] = 1700;
		datapoints[2][4] = 1200;
		datapoints[2][5] = 550;
		datapoints[2][6] = -1;
		datapoints[2][7] = 200;
		datapoints[2][8] = 1200;
		datapoints[2][9] = -1;
		
//		setTime(300);
		computeSchedule();
		
	
		for(int i=0; i<number_of_solvers; ++i)
		{
			if( allocated_time[i] > 0 )
				System.err.println( "c\t  run solver " + (i+1) + " for " +
						(allocated_time[i]*time_unit) + " seconds " + solved[i][allocated_time[i]] );
		}
	}
	
	
	public void testTimeAllocation2() {
		number_of_solvers = 3;
		ndatapoints = 10;
		datapoints = new double[3][10];
		datapoints[0][0] = -1;
		datapoints[0][1] = 50;
		datapoints[0][2] = 10;
		datapoints[0][3] = 15;
		datapoints[0][4] = -1;
		datapoints[0][5] = -1;
		datapoints[0][6] = -1;
		datapoints[0][7] = -1;
		datapoints[0][8] = -1;
		datapoints[0][9] = 10;
		
		datapoints[1][0] = 450;
		datapoints[1][1] = -1;
		datapoints[1][2] = -1;
		datapoints[1][3] = 100;
		datapoints[1][4] = -1;
		datapoints[1][5] = -1;
		datapoints[1][6] = -1;
		datapoints[1][7] = 350;
		datapoints[1][8] = -1;
		datapoints[1][9] = -1;
		
		datapoints[2][0] = -1;
		datapoints[2][1] = -1;
		datapoints[2][2] = 70;
		datapoints[2][3] = 70;
		datapoints[2][4] = -1;
		datapoints[2][5] = 50;
		datapoints[2][6] = -1;
		datapoints[2][7] = 200;
		datapoints[2][8] = 10;
		datapoints[2][9] = -1;
		
		computeSchedule();
		
	
		for(int i=0; i<number_of_solvers; ++i)
		{
			if( allocated_time[i] >= 0 )
				System.err.println( "c\t  run solver " + (i+1) + " for " +
						(allocated_time[i]*time_unit) + " seconds " + solved[i][allocated_time[i]] );
		}
	}
	
	
	public static void main(String[] args){
		ThreeSolversSchedule s = new ThreeSolversSchedule();
		s.testTimeAllocation2();
	}
	
}
