package cases;

import java.util.HashMap;
import java.util.Iterator;

import features.Feature;

/**
 * 
 * 
 * 
 *
 */
public class Case{
	
	private String caseID;

	/**
	 * A HashMap storing the <code>Features</code> of this <code>StemCase</code> indexed by feature names
	 */
	private HashMap<String, Feature> indexedFeatures;
	
	/**
	 * Constructs a <code>StemCase</code> object.
	 */
	public Case(String caseID) {
		this.caseID = caseID;
		indexedFeatures = new HashMap<String, Feature>();
	}
	
	/**
	 * Adds the specified <code>Feature</code> to this <code>Case</code>.
	 * @param feature the <code>Feature</code> to be added to this <code>Case</code>..
	 * 
	 */
	public void addFeature(Feature feature) {
		String name = feature.getName();
		indexedFeatures.put(name, feature);

	}

	/**
	 * Returns the <code>Features</code> corresponding to the specified name in this <code>Case</code>.
	 * @return the <code>Features</code> corresponding to the specified name in this <code>Case</code> or <code>null</code> if there
	 *         is no feature with the specified name.
	 * @param featureName
	 *            name whose associated feature is to be returned.
	 */
	public Feature getFeature(String featureName) {
		return indexedFeatures.get(featureName);
	}

	/**
	 * Returns an Enumeration of all the feature names held by the case
	 * @return names of all the <code>Features</code> held in the <code>Case</code>
	 */
	public Iterator<String> featureNames(){
		return indexedFeatures.keySet().iterator();
	}
	
	/**
	 * Returns the id of this <code>Case</code>.
	 * 
	 * @return the id of this <code>Case</code>.
	 */
	public String getCaseID() {
		return caseID;
	}
	/**
	 * Set the case ID
	 * @param caseID
	 */
	public void setCaseID(String caseID){
		this.caseID = caseID;
	}
	
	/**
	 * Returns the number of features in this case
	 * @return the number of features in this case
	 */
	public int getNumberOfFeatures() {
		return indexedFeatures.size();
	}
	
	/**
	 * Removes the specified feature from the case
	 * @param featureName name of the feature to remove
	 * @return the removed feature
	 */
	public Feature removeFeature(String featureName){
		return indexedFeatures.remove(featureName);
	}
	
	public Object clone(){
		Case clone = new Case(new String(this.getCaseID()));
		Iterator<String> it = this.featureNames();
		while(it.hasNext()){
			String featureName = it.next();
			Feature feat = (Feature) this.getFeature(featureName).clone();
			clone.addFeature(feat);
		}
		return clone;
	}

}
