package cases;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import features.SolverTimesFeature;

import similarity.SimilarityRanker;

import adaption.AdaptationAgent;
import adaption.SolverSolution;


public class CaseBasedReasoner{
	
	private AdaptationAgent adapter;
	private int k =3;
	private SimilarityRanker ranker;
	private boolean includeEqualSims = false;
	
	public CaseBasedReasoner( AdaptationAgent adapter, int k){
		this.adapter = adapter;
		this.k = k;
	}
	
	public CaseBasedReasoner(AdaptationAgent adapter, int k, boolean includeEqualSims ){
		this.adapter = adapter;
		this.k = k;
		this.includeEqualSims = includeEqualSims;
	}
	
	public void buildReasoner(Casebase casebase){
		ranker = new SimilarityRanker(casebase, true);
	}
	
	public List<Case> retrieveCases(Case query){
		List<Case> cases = ranker.getSimilarNeighbours(query, true);
		List<Case> theCases = new ArrayList<Case>();
		Iterator<Case> it = cases.iterator();
		for(int x=0;x<k;x++){
			theCases.add(it.next());
		}
	
		if(includeEqualSims){
			double sim = ranker.getSimilarities().get(theCases.get(theCases.size()-1));
			Case aCase = it.next();
			boolean notEqual = false;
			while(it.hasNext()&&!notEqual){
				double otherSim = ranker.getSimilarities().get((aCase));
				if(sim==otherSim){
					theCases.add(aCase);
					aCase = it.next();
				}else{
					notEqual = true;
				}
			}
			
		}
		return theCases;
	}
	
	public SolverSolution classify(Case query){
//		
//		System.out.println("c " + query.getCaseID());
//		System.out.print("c ");
//		for(int i = 0; i < query.getNumberOfFeatures(); ++i)
//			System.out.print(query.getFeature(featureName));
		
		Case temp = query;
		System.err.println("c CASE TO CLASSIFY:");
		System.err.println("c " + temp.getCaseID());
		Iterator<String> featureNames = temp.featureNames();
		while(featureNames.hasNext()){
			String name = featureNames.next();
			if(!name.equals("Times")){
				System.err.printf("c " + name + " > %10f | ", ((Double)
						(temp.getFeature(name).getValue())).doubleValue());
			}
		}
		System.err.println("c");
		SolverTimesFeature ftr = 
			(SolverTimesFeature)
			temp.getFeature("Times");
		Iterator<String> solvers = ftr.getValue().keySet().iterator();
		while(solvers.hasNext()){
			String temp_solver = solvers.next();
			System.err.println("c " + temp_solver + 
					" : " + ftr.getValue().get(temp_solver));
		}
		System.err.println("c\nc");
	
			
		
		List<Case> theCases = retrieveCases(query);
		
		return adapter.adaptCases(theCases,ranker.getSimilarities(), query);
	}
	
	public SimilarityRanker getRanker(){
		return ranker;
	}
	
	public SolverSolution classify(Case query, double time){
		
		List<Case> theCases = retrieveCases(query);
		
		return adapter.adaptCases(theCases,ranker.getSimilarities(), query);
	}

}
