package evaluate;

import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.Map;

import parsers.Parser;
import adaption.AdaptToSchedule;
import adaption.SolverScheduleSolution;

import cases.Case;
import cases.CaseBasedReasoner;
import cases.Casebase;
import features.SolverTimesFeature;

public class EvaluateReasoner{
	
	public static void runClassify(Casebase casebase , int k) throws IOException{
		PrintWriter out = new PrintWriter(
				new BufferedWriter(new FileWriter("/home/edom/h_errors")));
		
		double numRight = 0;
		double numInstances = 0;
		CaseBasedReasoner cbr = new CaseBasedReasoner(new AdaptToSchedule(),  k);
		cbr.buildReasoner(casebase);
		double schedule_sol = 0;
		double split_3_time = 0;
		double abscon = 0;
		double mistral = 0;
		double choco = 0;

		for(int x=0;x<casebase.size();x++){

			Case query = casebase.getCase(x);
			if(isValid((SolverTimesFeature)query.getFeature(Parser.SOLVER_TIMES_FEATURE))){

				numInstances++;
//				List<Case> cases = cbr.retrieveCases(query);
//				Case predicted = cases.get(1);
//				SolverTimesFeature predSolverFeat = (SolverTimesFeature)predicted.getFeature(Parser.SOLVER_TIMES_FEATURE);
				SolverTimesFeature querySolverFeat = (SolverTimesFeature)query.getFeature(Parser.SOLVER_TIMES_FEATURE);
				SolverScheduleSolution schedule = (SolverScheduleSolution)
				cbr.classify(query);
				Iterator<String> itr = schedule.getSolverAndTimes().keySet().iterator();
				while(itr.hasNext()){
					String name = itr.next();
					double time = schedule.getSolverAndTimes().get(name);
					if(querySolverFeat.getValue().get(name) < time){
						schedule_sol++;
						break;
					} else if (!itr.hasNext()){
						out.println("Problem: " + query.getCaseID());
						out.println("ID: " + x);
						out.flush();
					}
				}
				if(querySolverFeat.getValue().get("abscon") < 1800)
					abscon++;
				if(querySolverFeat.getValue().get("mistral") < 1800)
					mistral++;
				if(querySolverFeat.getValue().get("choco") < 1800)
					choco++;
				if(querySolverFeat.getValue().get("abscon") < 600)
					split_3_time++;
				else if(querySolverFeat.getValue().get("mistral") < 600)
					split_3_time++;
				else if(querySolverFeat.getValue().get("choco") < 600)
					split_3_time++;


				System.out.println("c Percentage correct: "+(numRight/numInstances*100));
				System.out.println("c Schedule: " + (schedule_sol/numInstances*100));
				System.out.println("c Abscon: " + (abscon/numInstances*100));
				System.out.println("c Mistral: " + (mistral/numInstances*100));
				System.out.println("c Choco: " + (choco/numInstances*100));
				System.out.println("c Split time " + (split_3_time/numInstances*100));
			}

//			if(isValid(predSolverFeat)&&isValid(querySolverFeat)){
//				//System.out.println("\nPredicted");
//				String solverRetrieved = getBestSolver((SolverTimesFeature)predicted.getFeature(Parser.SOLVER_TIMES_FEATURE));
//				//System.out.println("Actual");
//				String solverQuery = getBestSolver((SolverTimesFeature)query.getFeature(Parser.SOLVER_TIMES_FEATURE));
//				//System.out.println("Predicted: "+solverRetrieved+" Actual: "+solverQuery);
//				if(solverRetrieved.equals(solverQuery)){
//					numRight++;
//				}
//				numInstances++;
//			}else{
//				
//				//System.out.println("Bas cases");
//			}
		}
		
		System.out.println("c Percentage correct: "+(numRight/numInstances*100));
		System.out.println("c Schedule: " + (schedule_sol/numInstances*100));
		System.out.println("c Abscon: " + (abscon/numInstances*100));
		System.out.println("c Mistral: " + (mistral/numInstances*100));
		System.out.println("c Choco: " + (choco/numInstances*100));
		System.out.println("c Split time " + (split_3_time/numInstances*100));
		System.out.println("c Number excluded "+(casebase.size()-numInstances));
		System.out.println("c The % exluded is "+(casebase.size()-numInstances)/casebase.size()*100);
		out.close();
		System.out.println("Percentage solved: "+(numRight/numInstances*100));
		double numExcluded =  (casebase.size()-numInstances);
		System.out.println("Number excluded "+numExcluded);
		System.out.println("The % exluded is "+(numExcluded/casebase.size()*100));	
		
	}
	
	public static String getBestSolver(SolverTimesFeature feat){
		Map<String, Double> times = feat.getValue();
		String bestSolver = "";
		double bestTimes = Double.MAX_VALUE;
		Iterator<String> it = times.keySet().iterator();
		while(it.hasNext()){
			String solver = it.next();
			double value = times.get(solver).doubleValue();
			//System.out.println(solver+" time: "+value);
			if(value<bestTimes){
				bestSolver = solver;
				bestTimes = value;
			}
		}
		
		return bestSolver;
	}
	
	public static boolean isValid(SolverTimesFeature feat){
		Map<String, Double> times = feat.getValue();
		Iterator<String> it = times.keySet().iterator();
		while(it.hasNext()){
			if(times.get(it.next()).doubleValue()<1800.0){//TODO: change me
				return true;
			}
		}
		return false;
	}
	
	public static void main(String[] args){
		String testDir = "/home/edom/cpHydra/named_casebase";
		Casebase casebase = Parser.parseNamedFile(testDir);
		//String testDir = "CBR Data/Parser Output Data/named_casebase";
		//Casebase casebase = Parser.parseNamedFile(testDir);
		//NormaliseCasebase.normaliseCasebase(casebase, NormaliseCasebase.getMinMax(casebase));
		try {
			EvaluateReasoner.runClassify(casebase, 10);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			System.exit(1);
			e.printStackTrace();
		}
	}
	
	
	
	

}
