package evaluate;

import java.util.ArrayList;
import java.util.List;

import parsers.Parser;

import adaption.AdaptToSchedule;
import adaption.SolverScheduleSolution;

import similarity.ArrayEuclideanSimilarity;

import cases.Case;
import cases.CaseBasedReasoner;
import cases.Casebase;
import features.FeatureStructure;
import features.SolverTimesFeature;

public class TenFoldCrossValidation {
	
	private int time_available = 1800;
	
	private int total_solved;
	private int total_problems;
	private int total_left_out;
	private int static_solved; 
	
	private int total_abscon = 0;
	private int total_mistral = 0;
	private int total_bprolog = 0;
	private int total_buggy = 0;
	private int total_sugar = 0;
	
	private int total_split = 0;
	
	public TenFoldCrossValidation(){
		total_solved = 0;
		total_problems = 0;
		total_left_out = 0;
		static_solved = 0;
	}
	
	public void tenFoldCrossValidation(Casebase casebase, int k){
		ArrayList<ArrayList<Case>> cases = new ArrayList<ArrayList<Case>>();
		List<Case> total_cases = casebase.getCases();
		int fold_size = total_cases.size()/10;
//		System.out.println("c Fold Size: " + fold_size);
		for(int i = 0; i < 10; ++i){
			ArrayList<Case> temp_list = new ArrayList<Case>();
			for(int j = 0; j < fold_size; ++j)
				temp_list.add(total_cases.remove((int)
						(total_cases.size()*Math.random())));
			cases.add(temp_list);
		}
		for(int i = 0; i < 10; ++i){
			List<Case> test = cases.get(i);
			List<Case> training = new ArrayList<Case>();
			for(int j = 0; j < cases.size(); ++j)
				if(j != i)
					training.addAll(cases.get(j));
//			System.out.println("c test size: " + test.size());
//			System.out.println("c training size: " + training.size());
			evaluate(training, test, k);
		}
		
		System.out.println("c Total Solved: " + 
				(total_solved/(double)total_problems)*100);
		System.out.println("Total static: " + 
		(static_solved/(double)total_problems)*100);
		System.out.println("");
		System.out.println("Abscon: " + (total_abscon/(double)total_problems)*100);
		System.out.println("Mistral: " + (total_mistral/(double)total_problems)*100);
		System.out.println("Buggy: " + (total_buggy/(double)total_problems)*100);
		System.out.println("bprolog: " + (total_bprolog/(double)total_problems)*100);
		System.out.println("supar: " + (total_sugar/(double)total_problems)*100);
		System.out.println("");
		System.out.println("Split evenly: " + (total_split/(double)total_problems)*100);
		System.out.println("=======================");
		
		System.out.println("Total left out:" + this.total_left_out/((double)this.total_problems+total_left_out)*100);
	}
	
	public void evaluate(List<Case> training, List<Case> testing, int k){
		Casebase train = new Casebase("training", training, 
				new FeatureStructure[]{
				new FeatureStructure("Features", true, 
						new ArrayEuclideanSimilarity())});
		Casebase test = new Casebase("testing", testing, 
				new FeatureStructure[]{
				new FeatureStructure("Features", true, 
						new ArrayEuclideanSimilarity())});
		
//		CaseBasedReasoner cbr = new CaseBasedReasoner(new AdaptToSchedule(),
//				train.size());
//		cbr.buildReasoner(train);
//		SolverScheduleSolution static_sol =  
//			(SolverScheduleSolution) cbr.classify(train.getCase(0));
//		
//		cbr = new CaseBasedReasoner(new AdaptToSchedule(),k);
//		cbr.buildReasoner(train);
		
		int num_problems = 0;
		int solved = 0;
		int stat_solved = 0;
		int abscon = 0;
		int mistral = 0;
		int bprolog = 0;
		int buggy = 0;
		int sugar = 0;
		int split = 0;
		
		for(int i = 0; i < test.size(); ++i){
			Case query = test.getCase(i);
			if(EvaluateReasoner.isValid(
					(SolverTimesFeature)query.getFeature(
							Parser.SOLVER_TIMES_FEATURE))){
//				SolverScheduleSolution sol =  
//					(SolverScheduleSolution) cbr.classify(query);
//				if(EvaluateScheduler.isSolved(query, sol)){
//					solved++;
//				}
//				if(EvaluateScheduler.isSolved(query, static_sol)){
//					stat_solved++;
//				}
				//begin individual solvers
				if(((SolverTimesFeature)
						(query.getFeature(Parser.SOLVER_TIMES_FEATURE)
					    )).getValue().get("abscon") < time_available){
					abscon++;
				}
				if(((SolverTimesFeature)
						(query.getFeature(Parser.SOLVER_TIMES_FEATURE)
								)).getValue().get("mistral") < time_available){
					mistral++;
				}
				if(((SolverTimesFeature)
						(query.getFeature(Parser.SOLVER_TIMES_FEATURE)
								)).getValue().get("buggy") < time_available){
					buggy++;
				}
				if(((SolverTimesFeature)
						(query.getFeature(Parser.SOLVER_TIMES_FEATURE)
								)).getValue().get("bprolog") < time_available){
					bprolog++;
				}
				if(((SolverTimesFeature)
						(query.getFeature(Parser.SOLVER_TIMES_FEATURE)
								)).getValue().get("sugar") < time_available){
					sugar++;
				}
				//end individual solvers
				//begin split test
				if(((SolverTimesFeature)
						(query.getFeature(Parser.SOLVER_TIMES_FEATURE)
								)).getValue().get("abscon") < time_available/5){
					split++;
				}
				else if(((SolverTimesFeature)
						(query.getFeature(Parser.SOLVER_TIMES_FEATURE)
								)).getValue().get("mistral") < time_available/5){
					split++;
				}
				else if(((SolverTimesFeature)
						(query.getFeature(Parser.SOLVER_TIMES_FEATURE)
								)).getValue().get("buggy") < time_available/5){
					split++;
				}
				else if(((SolverTimesFeature)
						(query.getFeature(Parser.SOLVER_TIMES_FEATURE)
								)).getValue().get("bprolog") < time_available/5){
					split++;
				}
				else if(((SolverTimesFeature)
						(query.getFeature(Parser.SOLVER_TIMES_FEATURE)
								)).getValue().get("sugar") < time_available/5){
					split++;
				}
				//end split test
				num_problems++;
			}
		}
//		System.out.println("c Fold problems: " + num_problems);
//		System.out.println("c % solved: " + (solved/(double)num_problems)*100);
//		System.out.println("c % static: " + (stat_solved/(double)num_problems)*100);
		System.out.println("c left out: " + 
				((test.size()-num_problems)/(double)test.size())*100);
		total_solved += solved;
		total_problems += num_problems;
		total_left_out += test.size()-num_problems;
		static_solved += stat_solved;
		
		total_abscon += abscon;
		total_mistral += mistral;
		total_buggy += buggy;
		total_bprolog += bprolog;
		total_sugar += sugar;
		
		total_split += split;
	}

	public static void main(String[] args){
//		for(int i = 0; i < 10; ++i){
			Casebase cb = Parser.parseNamedFile("/home/edom/casebase2006");
			new TenFoldCrossValidation().tenFoldCrossValidation(cb, 10);
//			cb = Parser.parseNamedFile("/home/edom/casebase2006");
//		}
	}
}
