package features;


/**
 * This is a general representation of a feature
 * 
 */

public abstract class Feature{
	
	
	/**
	 * The name of this feature
	 */
	protected String name;

	/**
	 * Constructs a <code>Feature</code> Object with the specified featureName
	 * @param featureName the name of this feature
	 */
	protected Feature(String featureName) {
		this.name = featureName;
	}
	
	/**
	 * Returns the name of this <code>Feature</code>.
	 * @return the name of this <code>Feature</code>.
	 */
	public String getName() {
		return name;
	}
	
	 /**
     * Compares this Feature to the specified feature. 
     * @param   obj the object to compare this <code>Feature</code> against.
     * @return  <code>true</code> if the <code>Features </code>are equal;
     *          <code>false</code> otherwise.
     */
	public abstract boolean equals(Object obj);
		
	
	/**
	 * Returns a clone of this <code>Feature</code>.
	 * @return a clone of this <code>Feature</code>.
	 */
	public abstract Object clone();
	
	/**
	 * Sets the value of this feature to the specified value. The type of <code>value</code> will depend on the type of <code>Feature</code>.	
	 * 
	 * @param value the value to be set (a <code>String</code> if this is a simple feature or a <code>Feature</code> if this is a complex feature.
	 * @throws BadFeatureValueException if <code>newValue</code> is not in the expected Format, i.e. a <code>String</code> for a simple feature or a <code>Feature</code> for a complex feature.	 
	 */
	public abstract void setValue(Object value) throws BadFeatureValueException;
	
	/**
	 * Returns the value of this feature. The type will depend on the type of <code>Feature</code>.
	 * 
	 * @return the value of this feature.
	 */
	abstract public Object getValue();
	
	public Object getWeights(){
		return null;
	}
}
