package parsers;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Scanner;

import similarity.ArrayEuclideanSimilarity;

import cases.Case;
import cases.Casebase;
import features.DoubleFeature;
import features.FeatureStructure;
import features.SolverTimesFeature;



public class Parser {
	
	//public static String FEATURE_ROOT = "Feature: ";
	public static String CASE_NAME_ROOT = "Case: ";
	public static String SOLVER_TIMES_FEATURE = "Times";
	public static int NUMBER_FEATURES = 36;
	
	public static Casebase parseFile(String file){
		List<Case> cases = new ArrayList<Case>();
		int caseIndex =0;
		try {
			Scanner input = new Scanner(new FileInputStream(new File(file)));
			while(input.hasNext()){
				Case next_case = new Case(CASE_NAME_ROOT+caseIndex);
				HashMap<String, Double> solverTimes = 
					new HashMap<String, Double>();
				//double[] features = new double[NUM_FEATURES];
				int index = 0;
				String line = input.nextLine();
				Scanner line_scan = new Scanner(line);
				while(line_scan.hasNext()){
					if(index < NUMBER_FEATURES)
						next_case.addFeature(new DoubleFeature(
								Integer.toString(index++), line_scan.nextDouble()));
					else{
						solverTimes.put(line_scan.next(), line_scan.nextDouble());
					}
				}
				next_case.addFeature(new SolverTimesFeature(SOLVER_TIMES_FEATURE, solverTimes));
				cases.add(next_case);
				caseIndex++;
			}
			Casebase cb = new Casebase("training", cases, new FeatureStructure[]{
					new FeatureStructure("Features", true, 
							new ArrayEuclideanSimilarity())
			});
			return cb;
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public static Casebase parseNamedFile(String file){
		List<Case> cases = new ArrayList<Case>();
		try {
			Scanner input = new Scanner(new FileInputStream(new File(file)));
			while(input.hasNext()){
				Case next_case = new Case(input.next());
				HashMap<String, Double> solverTimes = 
					new HashMap<String, Double>();
				//double[] features = new double[NUM_FEATURES];
				int index = 0;
				String line = input.nextLine();
				Scanner line_scan = new Scanner(line);
				while(line_scan.hasNext()){
					if(index < NUMBER_FEATURES){
						next_case.addFeature(new DoubleFeature(
								Integer.toString(index++), line_scan.nextDouble()));
					}
					else{
						solverTimes.put(line_scan.next(), line_scan.nextDouble());
					}
				}
				next_case.addFeature(new SolverTimesFeature(SOLVER_TIMES_FEATURE, solverTimes));
				cases.add(next_case);
			}
			Casebase cb = new Casebase("training", cases, new FeatureStructure[]{
					new FeatureStructure("Features", true, 
							new ArrayEuclideanSimilarity())
			});
			return cb;
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		return null;
	}
}
