package similarity;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;

import cases.Case;
import cases.Casebase;
import features.FeatureStructure;


public class SimilarityRanker{
	
	
	private Casebase casebase;
	
	/**
	 * The feature structure
	 * 
	 */
	private FeatureStructure[] featureStructure;
	
	/**
	 * whether the similarity or dissimilarity measures are being used
	 * 
	 */
	private boolean dissimilarity = false;
	
	private HashMap<Case,Double> similarities;
	
	
	/**
	 * @param casebase that is used
	 */
	public SimilarityRanker(Casebase casebase){
		this.casebase = casebase;
		this.featureStructure = casebase.getCaseStructure();
	}
	
	public SimilarityRanker(Casebase casebase, boolean dissimilarity){
		this.casebase = casebase;
		this.featureStructure = casebase.getCaseStructure();
		this.dissimilarity = dissimilarity;
	}
	
	/**
	 * 
	 * @param queryCase
	 * @param includeQuery
	 * @return
	 */
	public List<Case> getSimilarNeighbours(Case queryCase, boolean includeQuery){

		List<Case> cases = new ArrayList<Case>();
		similarities = new HashMap<Case,Double>();
		for(int x=0; x<casebase.size(); x++){
			Case aCase = casebase.getCase(x);
			if(!(queryCase.equals(aCase) && !includeQuery)){
				double sim = getSimilarity(queryCase,aCase);
				similarities.put(aCase, sim);
				cases.add(aCase);
			}
		}
		Collections.sort(cases,new SimilarityComparitor(similarities, dissimilarity));
		return cases;
	}

	public double getSimilarity(Case queryCase, Case aCase){
		return ((ArrayEuclideanSimilarity)(
				featureStructure[0].getSimilarityMeasure())).
				getSimilarity(queryCase, aCase);
	}
	
	public HashMap<Case,Double> getSimilarities(){
		return similarities;
	}
	
	
	public class SimilarityComparitor implements Comparator<Case>{
		
		private HashMap<Case,Double> similarities;
		private boolean dissimilarity = false;
		
		public SimilarityComparitor(HashMap<Case,Double> similarities){
			this.similarities= similarities;
		}
		
		public SimilarityComparitor(HashMap<Case,Double> similarities, boolean dissimilarity){
			this.similarities= similarities;
			this.dissimilarity = dissimilarity;
		}
		
		public int compare(Case caseOne, Case caseTwo) {
			double CaseOneSim = similarities.get(caseOne);
			double CaseTwoSim = similarities.get(caseTwo);
			
			if(CaseOneSim>=CaseTwoSim){
				if(CaseOneSim>CaseTwoSim){
					if(dissimilarity){
						return 1;
					}else{
						return -1;
					}
				}
				return 0;
			}
			if(dissimilarity){
				return -1;
			}else{
				return 1;
			}
		}
	}
	
	

}
