package solver_execution;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import adaption.SolverScheduleSolution;



public class SolverManager {
	
	private ArrayList<String> error;
	private ArrayList<String> stdout;
	private String home;
	private Process p;
	
	public SolverManager(String path){
		this.home = path;
	}
	
	/**
	 * Runs the specified solver schedule
	 * @param sol Solver Schedule to be run
	 * @param path Path to instance to be solved
	 */
	public void run(SolverScheduleSolution sol, String path){
		HashMap<String, Integer> times = sol.getSolverAndTimes();
		
		Iterator<String> solvers = times.keySet().iterator();
		SolverRunningInformation info = new SolverRunningInformation(home); 
//		List<String> order = new ArrayList<String>();
		String solver;
//		while(solvers.hasNext()){
//			solver = solvers.next();
//			if(solver != null){
//				int index = 0;
//				while(index < order.size() && times.get(order.get(index)) <
//						times.get(solver))
//						;
//				order.add(index, solver);
//			}
//		}
//		
//		solvers = order.iterator();
//		while(solvers.hasNext()){
//			solver = solvers.next();
//			if(solver != null)
//				runFor(info.getCommand(solver), path, times.get(solver), 
//						solvers.hasNext());
//		}
		// then run the default solver until we are killed
//		if(times.get("mistral") != null && times.get("choco") == null){
//			if(times.get("abscon") == null)
//				runFor(info.getCommand("mistral"), path, times.get("mistral"), false);
//			else{
//				runFor(info.getCommand("abscon"), path, times.get("abscon"), true);
//				runFor(info.getCommand("mistral"), path, times.get("mistral"), false);
//			}
//		} else if(times.get("mistral") != null){
//			runFor(info.getCommand("mistral"), path, times.get("mistral"), true);
//			if(times.get("abscon") != null){
//				runFor(info.getCommand("abscon"), path, times.get("abscon"), true);
//			}
//			runFor(info.getCommand("choco"), path, times.get("choco"), false);
//		} else if(times.get("abscon") == null)
//			runFor(info.getCommand("choco"), path, times.get("choco"), false);
//		else{
//			runFor(info.getCommand("abscon"), path, times.get("abscon"), true);
//			runFor(info.getCommand("choco"), path, times.get("choco"), false);
//		}
		if(times.get("abscon") != null){
			runFor(info.getCommand("abscon"), path, times.get("abscon"), !(times.get("mistral") == null &&
						times.get("choco") == null));
		}
		if(times.get("mistral") != null)
			runFor(info.getCommand("mistral"), path, times.get("mistral"), (times.get("choco") != null));
		if(times.get("choco") != null)
			runFor(info.getCommand("choco"), path, times.get("choco"), false);
		
	}

	/**
	 * Runs the given solver on the given instance, returns an answer to the user
	 * if a solution is found.
	 * @param command Path to the script to run the solver
	 * @param arg Path the the XML instance 
	 * @param integer Amount of seconds solver is to be run for
	 * @param last true if this is the last command
	 */
	private void runFor(String command, String arg, final Integer integer, 
			boolean not_last) {
		if(command != null){
			try {
				System.err.println("c Starting solver: " + command);
				System.err.println("c On Instance: " + arg);
				System.err.println("c Running for: " + integer + " seconds");
				stdout = new ArrayList<String>();
				error = new ArrayList<String>();

				ProcessBuilder pb;
				//if(not_last)
					pb = new ProcessBuilder(command, arg, 
							new Integer(integer).toString());
				//else 
				//	pb = new ProcessBuilder(command, arg);
				p = pb.start();

				String s;					
				BufferedReader stdInput = new BufferedReader(new 
						InputStreamReader(p.getInputStream()));

				BufferedReader stdError = new BufferedReader(new 
						InputStreamReader(p.getErrorStream()));

				while ((s = stdInput.readLine()) != null) {
					stdout.add(s);
				}

				while ((s = stdError.readLine()) != null) {
					error.add(s);
				}
			} catch (Throwable t){

			}
			if(checkIfSolved()){
				for(int i = 0; i < stdout.size(); ++i)
					System.out.println(stdout.get(i));
				System.exit(0);
			} 
		}
	}

	/**
	 * @return True is the solver just ran solved the problem
	 */
	private boolean checkIfSolved() {
		for(int i = stdout.size()-1; i >= 0; --i){
			if(stdout.get(i).contains("SAT"))
				return true;
		}
		return false;
	}

	/**
	 * Test method to see if the timeout is working
	 */
	public void test(){
		System.out.println("c Starting test");
		runFor("/home/edom/CSP_Solvers/solver/mistral",
				"/home/edom/series-12.xml", 10000, false);
		System.out.println("c test done");
	}

	/**
	 * Test main, to see if the timeout is working
	 * @param args
	 */
	public static void main(String[] args){
		new SolverManager("path").test();
	}
}
